
function demo_incurvpolygon_cheb(example)

%--------------------------------------------------------------------------
% Object:
%
% Demo that tests an indomain routine on Jordan type domains "U" whose
% boundary "dU" is defined by two chebfuns "Fx", "Fy", such that each point
% of "dU" is represented as "(Fx(t),Fy(t))" for "t" in an interval "intv".
%--------------------------------------------------------------------------
% Input:
%
% example: parameter that chooses a certain domain.
%
%     case 1 % Disk
%     case 2 % Lune
%     case 3 % Union of two disks
%     case 4 % Intersection of two disks
%     case 5 % Cardioid (default)
%     case 6 % Deltoid
%     case 7 % Bicorn
%     case 8 % Ranuncoloid
%--------------------------------------------------------------------------
% Dates:
% Built: October 01, 2020.
% Last update: October 10, 2020.
%--------------------------------------------------------------------------

% .............................. Defaults .................................

if nargin < 1, example=5; end
pts_card=10000; % Pointset cardinality
do_plot=1; % 0: no plot 1: plot boundary and nodes.


% .............................. Domain ...................................

[Fx,Fy,str_domain]=chebfun_boundary(example);


% .............................. Pointset .................................

% As pointset we use Halton points in the smaller square "rect" with sides
% parallel to the "x" and "y" axes.

rect=[min(Fx) max(Fx) min(Fy) max(Fy)];
p=haltonset(2);
k0=0; % Halton points shift
X=rect(1)+(rect(2)-rect(1))*p(k0+1:k0+pts_card,1);
Y=rect(3)+(rect(4)-rect(3))*p(k0+1:k0+pts_card,2);
P=[X(:) Y(:)];


% .............................. Indomain .................................
tic;
[in,mon_box,singular_points,intv,Fx,Fy]=incurvpolygon_cheb(P,Fx,Fy);
cpus=toc;
pts_in=find(in == 1); % Indices of points inside the domain.

% ............................ Statistics .................................

str_dom_ext=strcat('(',str_domain,')');
fprintf('\n \t .........................................................');
fprintf('\n \t EXAMPLE          : %2.0f ',example); disp(str_dom_ext);
fprintf('\n \t POINTS           : %6.0f',length(X));
fprintf('\n \t POINTS IN DOMAIN : %6.0f',length(pts_in));
fprintf('\n \t CPUTIME PROCESS  : %1.1e',cpus);
fprintf('\n \t .........................................................');
fprintf('\n \n');

% ........................ Plot domain and points .........................

if do_plot == 1
    
    close all; clf;
    
    % .......... Figure 1 ..........
    plot_figure(Fx,Fy,P,P(pts_in,:),'Indomain routine',...
        {'boundary','outside domain','inside domain'});
    
end










function [Fx,Fy,str_domain]=chebfun_boundary(example)

%--------------------------------------------------------------------------
% Object:
% Definition of the chebfuns "Fx", "Fy", defined on the same interval
% "intv", so that each point "P" of the boundary is described as
% "P=(Fx(t),Fy(t))", with "t" in "intv".
%--------------------------------------------------------------------------
% Input:
% example: parameter that chooses a certain domain.
%
%     case 1 % Disk
%     case 2 % Lune.
%     case 3 % Union of two disks.
%     case 4 % Intersection of two disks.
%     case 5 % Cardioid.
%     case 6 % Deltoid
%     case 7 % Bicorn
%     case 8 % Ranuncoloid
%--------------------------------------------------------------------------
% Output:
%
% Fx,Fy: chebfun functions so that each point P of the boundary is
%    described as (Fx(t),Fy(t))
%
% str_domain: string with the name of the domain.
%--------------------------------------------------------------------------

switch example
    
    case 1 % Disk.
        str_domain='Unit Disk';
        Fx=chebfun('cos(t)',[0 2*pi],'splitting','on');
        Fy=chebfun('sin(t)',[0 2*pi],'splitting','on');
        
    case 2 % Lune.
        str_domain='Lune';
        Fx=chebfun({'0.5*(cos(t)+1)','0.5*cos((3*pi/2)-t)'},...
            [-pi/2 pi 3*pi/2],'splitting','on');
        Fy=chebfun({'0.5*(sin(t)+1)','0.5*sin((3*pi/2)-t)'},...
            [-pi/2 pi 3*pi/2],'splitting','on');
        
    case 3 % Union of two disks.
        str_domain='Union of two disks';
        Fx=chebfun({'cos(t)+sqrt(2)/2','cos(t-pi/2)-sqrt(2)/2'},...
            [-3*pi/4 3*pi/4 9/4*pi]);
        Fy=chebfun({'sin(t)','sin(t-pi/2)'}, ...
            [-3*pi/4 3*pi/4 9/4*pi],'splitting','on');
        
    case 4 % Intersection of two disks.
        str_domain='Intersection of two disks';
        Fx=chebfun({'cos(t)-sqrt(2)/2+1/2','cos(t+pi/2)+1/2+sqrt(2)/2'},...
            [-pi/4 pi/4 3/4*pi]);
        Fy=chebfun({'1/2+sin(t)','1/2+sin(t+pi/2)'}, ...
            [-pi/4 pi/4 3/4*pi],'splitting','on');
        
    case 5 % Cardioid.
        str_domain='Cardioid';
        Fx=chebfun('cos(t).*(1-cos(t))+1',...
            [0 2*pi],'splitting','on');
        Fy=chebfun('sin(t).*(1-cos(t))',...
            [0 2*pi],'splitting','on');
        
    case 6 % Deltoid (a=3, b=1 in Wolfram MathWorld definition).
        str_domain='Deltoid';
        Fx=chebfun('2*1*cos(t)+1*cos(2*t)',...
            [0 2*pi],'splitting','on');
        Fy=chebfun('2*1*sin(t)-1*sin(2*t)',...
            [0 2*pi],'splitting','on');
        
    case 7 % Bicorn
        str_domain='Bicorn';
        Fx=chebfun('sin(-t)',[-pi pi],'splitting','on');
        Fy=chebfun('( (cos(-t)).^2 .*(2+cos(-t)) ) ./ (3+(sin(-t)).^2)',...
            [-pi pi],'splitting','on');
        
    case 8 % Ranuncoloid
        str_domain='Ranuncoloid';
        Fx=chebfun('7*cos(t)-cos(7*t)',[0 2*pi],'splitting','on');
        Fy=chebfun('7*sin(t)-sin(7*t)',[0 2*pi],'splitting','on');
        
end










function plot_figure(Fx,Fy,xyw,xyw_c,title_str,legend_cell_str)

%--------------------------------------------------------------------------
% Object:
% This routine plots the boundary of a chebfun-curvilinear domain defined
% by "Fx and "Fy" as well as the nodes of the pointsets defined by "xyw"
% and "xyw_c".
%--------------------------------------------------------------------------
% Input:
% Fx, Fy: chebfuns defining the boundary of the domain as "(Fx(t),Fy(t))".
% xyw: M x 3 matrix defining cubature nodes and weights of a first rule.
% xyw_c: M x 3 matrix defining cubature nodes and weights of a second rule.
% title_str: string used in the title.
% legend_cell_str: cell containing the legend strings.
%--------------------------------------------------------------------------

% ....................... Trouble shooting ................................

if nargin < 4, xyw_c=[]; end
if nargin < 5, title_str=[]; end
if nargin < 6, legend_cell_str={}; end

% ..................... Chebfuns intervals ................................

intv=get(Fx,'ends'); intv=[intv(1) intv(end)];

% ..................... Plot boundary and points ..........................

axis equal; hold on; title(title_str);

% Plot boundary of the domain
t=linspace(intv(1),intv(2),1000); plot(Fx(t),Fy(t),'k-','LineWidth',4);

% Plot nodes of the first rule
if size(xyw,1) >0
    plot(xyw(:,1),xyw(:,2),'r+', 'MarkerSize',2);
end

% Plot nodes of the second rule
if size(xyw_c,1) >0
    plot(xyw_c(:,1),xyw_c(:,2),'g+', 'MarkerSize',2);
end

% Legend
switch length(legend_cell_str)
    case 1
        legend(legend_cell_str{1});
    case 2
        legend(legend_cell_str{1},legend_cell_str{2});
    case 3
        legend(legend_cell_str{1},legend_cell_str{2},legend_cell_str{3});
end

hold off;





