
function [in,mon_box,singular_points,intv,Fx,Fy,Fx1,Fy1]=...
    incurvpolygon_cheb(P,Fx,Fy,intv,Fx1,Fy1)

%--------------------------------------------------------------------------
% OBJECT:
% This routine checks if a point is inside a chebfun-curvilinear
% region whose boundary is defined by the vector chebfun Fx and Fy.
% It also detects problematic points as x-vertical segments and vertical
% cusps.
%--------------------------------------------------------------------------
% INPUT:
% P: points to test, defined as an N x 2 matrix.
%
% Fx: is a chebfun defining the "x=x(t)" parametric equation that describes
%      the boundary.
%
% Fy: is a chebfun defining the "y=y(t)" parametric equation that describes
%      the boundary.
%
% intv: it is a "1 x 2" vector; the boundary consists of "(Fx(t),Fy(t))"
%     with "t" in  "[intv(1) intv(2)]".
%
% Fx1: it is a chebfun defining the derivative of "Fx".
%
% Fy1: it is a chebfun defining the derivative of "Fy".
%--------------------------------------------------------------------------
% OUTPUT:
% in: the i-th component is
%
%     1 if P(i,:) is inside the domain,
%     0 if P(i,:) is inside the domain.
%
% mon_box: it is a matrix "M x 6" describing the "M" monotone boxes; the
%      k-th row has a form "[a b c d t0 t1]" and the monotone box is the
%      rectangle "[a,b] x [c,d]" and contains the portion of boundary
%      "(x(t),y(t))" with "t" belonging to "[t0,t1]".
%
% singular_points: vector "L x 1" containing the curvilinear abscissa "t"
%      of problematic points "singular_points", i.e. those "t" for which
%      the boundary (Fx(t),Fy(t)) "starts" having a vertical segment or in
%      which there is a vertical cusp, i.e. the derivative of "Fx" changes
%      sign.
%
% intv: it is a "1 x 2" vector; the boundary consists of "(Fx(t),Fy(t))"
%     with "t" in  "[intv(1) intv(2)]".
%
% Fx: it is a chebfun defining the "x=x(t)" parametric equation that
%     describes the boundary.
%
% Fy: it is a chebfun defining the "y=y(t)" parametric equation that
%     describes the boundary.
%
% Fx1: it is a chebfun defining the derivative of "Fx".
%
% Fy1: it is a chebfun defining the derivative of "Fy".
%--------------------------------------------------------------------------
% SUBROUTINES (directly called):
% 1. incurvpolygon_cheb_preprocess
% 2. indomain
%--------------------------------------------------------------------------
% Initial version: September 21, 2020.
% Revised version: October 9, 2020.
%--------------------------------------------------------------------------
%% Copyright (C) 2020- Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date:  September 21, 2020 - October 09, 2020.
%--------------------------------------------------------------------------

% .......... Compute chebfun objects / troubleshooting ....................

if isa(Fx, 'chebfun') & nargin < 4
    chebfun_Fx1_ends=get(Fx,'ends');
    intv=[min(chebfun_Fx1_ends) max(chebfun_Fx1_ends)];
end

if isa(Fx, 'chebfun') & size(intv,1) == 0
    chebfun_Fx1_ends=get(Fx,'ends');
    intv=[min(chebfun_Fx1_ends) max(chebfun_Fx1_ends)];
end

if isa(Fx, 'function_handle')
    Fx0=Fx;
    Fx=chebfun(Fx,intv,'splitting','on');
end

if isa(Fy, 'function_handle')
    Fy0=Fy;
    Fy=chebfun(Fy,intv,'splitting','on');
end

if isa(Fx, 'chebfun') & nargin < 5
    Fx1=diff(Fx);
end

if isa(Fx, 'chebfun') & nargin < 6
    Fy1=diff(Fy);
end

% .................. Compute monotone boxes ...............................
[mon_box,singular_points]=...
    incurvpolygon_cheb_preprocess(Fx,Fy,Fx1,Fy1,intv);


% ....................... Indomain routine ................................
in=indomain(P,Fx,Fy,mon_box,Fx1);










function [mon_box,singular_points]=...
    incurvpolygon_cheb_preprocess(Fx,Fy,Fx1,Fy1,intv)

%--------------------------------------------------------------------------
% OBJECT:
% This routine prepares the monotone boxes, fundamental for the indomain
% process.
% It detects problematic points "singular_points", i.e. those "t" for which
% the boundary (Fx(t),Fy(t)) "starts" having a vertical segment or in which
% there is a vertical cusp, i.e. the derivative of "Fx" changes sign.
% These points should be considered with care since they are difficult to
% treat by the crossing theorem, fundamental for the indomain routine.
%--------------------------------------------------------------------------
% INPUT:
% Fx: it is a chebfun defining the "x=x(t)" parametric equation that
%     describes the boundary.
% Fy: it is a chebfun defining the "y=y(t)" parametric equation that
%     describes the boundary.
% Fx1: it is a chebfun defining the derivative of "Fx".
% Fy1: it is a chebfun defining the derivative of "Fy".
% intv: it is a "1 x 2" vector; the boundary consists of "(Fx(t),Fy(t))"
%     with "t" in  "[intv(1) intv(2)]".
%--------------------------------------------------------------------------
% OUTPUT:
% mon_box: it is a matrix "M x 6" describing the "M" monotone boxes; the
%      k-th row has a form "[a b c d t0 t1]" and the monotone box is the
%      rectangle "[a,b] x [c,d]" and contains the portion of boundary
%      "(x(t),y(t))" with "t" belonging to "[t0,t1]".
% singular_points: vector "L x 1" containing the curvilinear abscissa "t"
%      of problematic points "singular_points", i.e. those "t" for which
%      the boundary (Fx(t),Fy(t)) "starts" having a vertical segment or in
%      which there is a vertical cusp, i.e. the derivative of "Fx" changes
%      sign.
%--------------------------------------------------------------------------
% SUBROUTINES (directly called):
% 1. box_analysis
%--------------------------------------------------------------------------
% Initial version: September 21, 2020.
% Revised version: October 4, 2020.
%--------------------------------------------------------------------------

% .......................... Initial settings .............................

% Each box is subdivided in "Nsub" sub-boxes, for improving the performance
% of the indomain procedure and the default "Nsub=210" seems a good choice.
% In view of the singular points we first compute the basic boxes, in which
% the boundary is defined by parametric abscissae "t" ranging in "[t0,t1]".
% We subdivide "[t0,t1]" in "Nsub" equispaced subintervals and compute a
% new set of smaller monotone boxes.

Nsub=210;



%  ...................... Determine monotone boxes ........................


% ..... Computing vertical singular points (vertical segments) .....

% Chebfun approximates functions via piecewise polynomials. The only chance
% that in a "maximal" interval "[t(k) t(k+1)]" (i.e. [t(k)-eps t(k+1)+eps]
% is not iden% tically zero) the chebfun function is constantly null is
% that after the "splitting on" it provides "[t(k) t(k+1)]" as end-points
% and there all Chebyshev coefficients are 0.
%
% To determine difficult abscissae "u", i.e. the boundary is somewhere
% a vertical segment made by points "P(u,y)", with "y" belonging to
% "[y0,y1]", that may have provide infinite crossing number for any
% "P(u,y)" where "y > y0", it is enough to check if in a subinterval of
% definition of "Fx1", such an function is null, i.e. all Chebyshev
% coefficients are 0.
%
% The variable "sing_pts_vert" contains the values "t" for which the
% boundary has a "vertical singular point" at "(Fx(t),Fy(t))", i.e there is
% a "vertical segment" with abscissa "Fx(t)", as previously described.
%
% After this vertical segment there is however a local turn, and we store
% such values in "sing_pts_cusps_0".
%
% There is no need of analise if the interval extrema "intv(1)" and
% "intv(end)" are singular points since they can only be at most turning
% points (i.e. with different sign in x-derivatives).

chebfun_Fx1_ends=get(Fx1,'ends');
chebfun_Fx1_coeffs=get(Fx1,'coeffs');

sing_pts_vert=[];
sing_pts_cusps_0=[];

for k=1:length(chebfun_Fx1_ends)-1
    if iscell(chebfun_Fx1_coeffs) == 1
        chebfun_coeffs_L=chebfun_Fx1_coeffs{k};
    else
        chebfun_coeffs_L=chebfun_Fx1_coeffs(k);
    end
    if 1 && all(chebfun_coeffs_L == 0)
        sing_pts_vert=[sing_pts_vert; chebfun_Fx1_ends(k)];
        sing_pts_cusps_0=[sing_pts_cusps_0; chebfun_Fx1_ends(k+1)];
    end
end





% ..................... Computing vertical cusps ..........................

% We consider here "cusps" due to the change of sign of the
% derivative of "Fx1" (i.e. "vertical-cusps").

% Computing potential cusps by "Fx1_roots".

Fx1_roots=roots(Fx1);

% Establishing change of sign of "Fx1" at "Fx1_roots".
%
% Here we take into account the sign of derivative of "Fx1" in the
% interval before "Fx1_roots(k)" and in the successive one, to
% establish the change sign.
%
% To this purpose, we compute the derivative at the midpoint
% "test_pts_tp(k,:)" of any subinterval determined by "chebfun_extrema" and
% "Fx1_roots". If this value is not null, then we are able to see
% if there is a "vertical change of direction", i.e. a "cusp-point".
%
% The variable "sing_pts_cusps" contains the values "t" for which the
% boundary has a cusp-point at "(Fx(t),Fy(t))".

chebfun_extrema=get(Fx,'ends');
intv_tp=union(chebfun_extrema,Fx1_roots);

if size(intv_tp,1) > 2 % More than one interval
    test_pts_tp=(intv_tp(1:end-1)+intv_tp(2:end))/2;
    sign_test_pts_tp=sign(Fx1(test_pts_tp));
    % Points with changes of sign for "Fx1" (i.e. "cusps").
    i_sign_changes=...
        find(sign_test_pts_tp(1:end-1).*sign_test_pts_tp(2:end) == -1);
    sing_pts_cusps=intv_tp(i_sign_changes+1);
else
    sing_pts_cusps=[];
end

% The initial and final value of the interval may provide a cusp-point.
% We check this instance.

if (sign(Fx1(intv(1))*Fx1(intv(2))) == -1)
    sing_pts_cusps(end+1)=intv(1);
end



% .......... Relevant parameters "t" for defining monotone boxes ..........

% Monotone boxes are determined by "t" that:
%
% * are chebfun "Fx" end-points: "chebfun_extrema"
% * determine vertical "singular_points": "sing_pts_vert"
% * determine "cusp-points": "sing_pts_cusps"
% * determine "points" after vertical "singular-points": "sing_pts_cusps_0"
%
% Some of these parameters "t" may be in more than one item above. The
% Matlab built-in function "union" sorts these values, avoiding
% repetitions.

% Make boxes considering also the roots of "Fy1" (this needs to well define
% a box via minima and maxima in each direction).

Fy1_roots=roots(Fy1);

singular_points=union(sing_pts_vert,sing_pts_cusps);

relevant_points=union(sing_pts_cusps_0,Fy1_roots);
relevant_points=relevant_points(:);

tbox=union(singular_points,relevant_points);
tbox=union(tbox,chebfun_extrema);

% Eliminate micro-boxes (they are typically numerical boxes generated
% by numerical errors).

tbox0=eliminate_duplicates(chebfun_extrema,tbox);

tbox=[tbox0(1)];
for k=1:length(tbox0)-1
    t0=tbox0(k); t1=tbox0(k+1);
    tL=linspace(t0,t1,Nsub+1);
    tbox=[tbox; tL(2:end)'];
end




% ............. Computing "monotone boxes" from "tbox" ....................

% We compute now the monotone boxes [xm,xM] x [ym,yM].

Fx_tbox=Fx(tbox); Fy_tbox=Fy(tbox);

Fx_tbox_mat=[Fx_tbox(1:end-1) Fx_tbox(2:end)];
xm=min(Fx_tbox_mat,[],2); xM=max(Fx_tbox_mat,[],2);

Fy_tbox_mat=[Fy_tbox(1:end-1) Fy_tbox(2:end)];
ym=min(Fy_tbox_mat,[],2); yM=max(Fy_tbox_mat,[],2);

% Fix possible box numerical problems (micro-holes or micro-overlapping
% boxes could appear due to not exact arithmetic).
% It is a main issue since some points could be erroneously classified.

xM=eliminate_duplicates2(xM,xm);
yM=eliminate_duplicates2(yM,ym);

[xm,~,im] = uniquetol(xm,10^(-13)); xm=xm(im);
[xM,~,iM] = uniquetol(xM,10^(-13)); xM=xM(iM);
[ym,~,im] = uniquetol(ym,10^(-13)); ym=ym(im);
[yM,~,iM] = uniquetol(yM,10^(-13)); yM=yM(iM);

tm=tbox(1:end-1); tM=tbox(2:end);

mon_box=[xm xM ym yM tm tM];










function set2out=eliminate_duplicates(set1,set2)

%--------------------------------------------------------------------------
% OBJECT:
% This routine keeps the elements in "set1" and deletes those in "set2"
% that are too close.
% Next, it computes the union of "set1" with the purget set.
% We cannot use "uniquetol" because we intend to be sure to use points in
% "set1".
%--------------------------------------------------------------------------
% INPUT:
% set1: vector with M components
% set2: vector with N components
%--------------------------------------------------------------------------
% OUTPUT:
% set2out: vector containing the elements in "set1" and deletes those in
%       "set2" that are too close (i.e. their distance is inferior to the
%       tolerance "tol=10^(-12)" and next, it computes the union of "set1"
%       with the purget set.
%--------------------------------------------------------------------------
% Initial version: October 3, 2020.
% Revised version: October 4, 2020.
%--------------------------------------------------------------------------

tol=10^(-12);
set2out=set2;
for k=1:length(set1)
    el1=set1(k);
    iL=find(abs(el1-set2out) > tol);
    set2out=set2out(iL);
end

set2out=union(set1,set2out);










function set1out=eliminate_duplicates2(set1,set2)


%--------------------------------------------------------------------------
% OBJECT:
% This routine substitutes the elements in "set1" with the closer one in
% "set2" if their distance is below a threshold.
%--------------------------------------------------------------------------
% INPUT:
% set1: vector with M components.
% set2: vector with N components.
%--------------------------------------------------------------------------
% OUTPUT:
% set1out: vector substituting the elements in "set1" with the closer one
% in "set2", if their distance is below a threshold.
%--------------------------------------------------------------------------
% Initial version: October 4, 2020.
% Revised version: October 4, 2020.
%--------------------------------------------------------------------------

tol=10^(-12);
set1out=set1;
for k=1:length(set1)
    [valmin,imin]=min(abs(set1(k)-set2));
    if valmin < tol
        set1out(k)=set2(imin);
    end
end










function in=indomain(P,Fx,Fy,boxV,Fx1)

%--------------------------------------------------------------------------
% OBJECT:
% This routine checks if a point is inside or on a chebfun-curvilinear
% region whose boundary is defined by the vector chebfun Fx and Fy.
%--------------------------------------------------------------------------
% INPUT:
% P: points to test, defined as an N x 2 matrix.
%
% Fx: is a chebfun defining the "x=x(t)" parametric equation that describes
%      the boundary.
%
% Fy: is a chebfun defining the "y=y(t)" parametric equation that describes
%      the boundary.
%
% boxV: S x k vector, whose i-th component is a spline defines the
%      properties of the i-th box of the boundary, i.e.
%
%                          [xm,xM,ym,yM]
%
% Fx1: it is a chebfun defining the derivative of "Fx".
%--------------------------------------------------------------------------
% OUTPUT:
% in: the i-th component is
%
%     1 if P(i,:) is inside the domain,
%     0 if not inside,
%     NaN in case of doubts and other winding algorithms must be used
%--------------------------------------------------------------------------
% SUBROUTINES (directly called):
% 1. box_analysis
%--------------------------------------------------------------------------
% Initial version: November 26, 2019.
% Revised version: September 21, 2020.
%--------------------------------------------------------------------------


% ......................... general study .................................

% We check if:
% a) a point P(k,:) is inside some monotone box and in this case
%    we perform some further analysis. This happens if the k-th column of
%    the variable "itestL" is not equal to the vector with all the
%    components equal to zero. The components equal to "1" say to what
%    boxes such point belong.
% b) a point P(k,:) is outside any monotone box. This happens if the k-th
%    column of the variable "itestL" is equal to the vector with all
%    the components equal to zero. In this case by its crossing number
%    "crossings(k)" we check if it is inside or outside the
%    chebfun-curvilinear polygon by means of the crossing theorem.
%
% Note: observe that "X" and "Y" are row vectors, while "a", "b", "c", "d"
%    are column vectors.
%
% Example:
% a=[3; 5; 7]; b=[4; 6; 8]; c=[1; 2; 5]; d=[3; 5; 9];
% X=3.5; Y=2.5; % The point belongs to the first box.
% iboxes=(X > a & X <=b & Y >= c);
% itestL=(iboxes & Y <= d);
% iregL=(iboxes & Y > d);
%
% as result:
%
% iboxes = [1 0 0]'
% itestL = [1 0 0]'
% iregL  = [0 0 0]'

X=(P(:,1))'; Y=(P(:,2))';
a=boxV(:,1); b=boxV(:,2); c=boxV(:,3); d=boxV(:,4);
iboxes=(X > a & X <=b & Y >= c);
itestL=(iboxes & Y <= d);
iregL=(iboxes & Y > d);
crossings=sum(iregL,1);


% ..................... study points inside boxes .........................


% Indices of the points requiring some special analysis (they are inside
% monotone boxes, even in more than one).
further_analysis_index=find(sum(itestL,1) > 0);

on=NaN*ones(size(X));
for jj=1:length(further_analysis_index)
    Pindex=further_analysis_index(jj);
    PL=P(Pindex,:);
    itestPL0=itestL(:,Pindex);
    index_testPL=find(itestPL0 > 0); % Detecting boxes containing PL
    
    final_value0=box_analysis(PL,index_testPL,boxV,Fx,Fy,Fx1);
    crossings(Pindex)=crossings(Pindex)+final_value0;
end
in=rem(crossings,2);










function final_value=box_analysis(P,itest,boxV,Fx,Fy,Fx1)

%--------------------------------------------------------------------------
% OBJECT:
% This routine checks if a point "P" that belongs to some boxes is "below",
% "above" or "on" the portion of boundary belonging to each monotone box in
% consideration, defined by "boxV(itest,:)".
% If "P" it not "on" any box, it says how many times the vertical straight
% line below "P" intersect the boundary.
%--------------------------------------------------------------------------
% INPUT:
% P: point to test, defined as an 1 x 2 matrix.
%
% itest: index of the box in the set described by "boxV", to be studied; if
%     this vector is not empty then the point P belongs to the boxes
%     represented by "boxV(itest,:)".
%
% boxV: S x k vector, whose i-th component is a spline defines the
%      properties of the i-th box of the boundary, i.e.
%
%                          [xm,xM,ym,yM,tm,tM]
%
% Fx: chebfun defining the "x=x(t)" parametric equation that defines the
%     boundary.
%
% Fy: chebfun defining the "y=y(t)" parametric equation that defines the
%     boundary.
%
% Fx1: chebfun defining the derivative of "Fx".
%
%--------------------------------------------------------------------------
% OUTPUT:
% final_value: crossing value (only in the boxes to which "P(k,:)"
%     belongs).
%--------------------------------------------------------------------------
% Attached routines:
% 1. newton
%--------------------------------------------------------------------------
% Initial version: September 21, 2020.
% Revised version: October 4, 2020.
%--------------------------------------------------------------------------


% ........................ Some troubleshooting ...........................

if nargin >= 6 % derivative is available
    root_method=1;
else % derivative is not available
    root_method=1;
end

tol=10^(-11);
final_value=0;

% .................... Analysis of special cases ..........................

boxVP=boxV(itest,:);

switch root_method
    case 1 % Using Chebfun "roots".
        tB_all=roots(Fx-P(1));
        [~,itP]=find( (tB_all'>=boxVP(:,5) & tB_all'<=boxVP(:,6)) == 1);
        yB_all=Fy(tB_all(itP));
    case 2 % Using Newton-Raphson.
        yB_all=[];
        for k=1:size(boxVP)
            t0=(boxVP(k,5)+boxVP(k,6))/2;
            f=Fx-P(1);
            yBL=newton(f,Fx1,t0,tol,10);
            yB_all=[yB_all; Fy(yBL)];
        end
end

final_value=length(find(yB_all < P(2)));










function [x,iter]=newton(f,df,x,tol,itmax)

%----------------------------------------------------------------
% Object:
% Newton-Raphson method for solving "f(x)=0".
%----------------------------------------------------------------
% Input:
% f: function defining the nonlinear equation "f(x)=0"
% df: "f" derivative
% x: initial guess
% tol: tolerance stopping criterion
% itmax: max number of iteration
%----------------------------------------------------------------
% Output:
% x: approximation
% iter:  number of iterations
%----------------------------------------------------------------

step=tol+1;
iter=0;

while (abs(step) > tol) & (iter < itmax)
    
    fx=feval(f,x);
    if fx == 0, return; end
    dfx=feval(df,x);
    if dfx == 0
        fprintf('\n \t Warning: Issue in Newton method: null derivative');
        return;
    end
    
    iter=iter+1;
    step=-fx/dfx;
    x=x+step;
    
end

if (iter >=itmax)
    fprintf('\n \t Warning: too many iterations in Newton method.')
end

